from spaceone.api.repository.v1 import dashboard_template_pb2, dashboard_template_pb2_grpc
from spaceone.core.pygrpc import BaseAPI


class DashboardTemplate(BaseAPI, dashboard_template_pb2_grpc.DashboardTemplateServicer):

    pb2 = dashboard_template_pb2
    pb2_grpc = dashboard_template_pb2_grpc

    def register(self, request, context):
        params, metadata = self.parse_request(request, context)
        with self.locator.get_service('DashboardTemplateService', metadata) as template_svc:
            template_data = template_svc.register(params)
            return self.locator.get_info('DashboardTemplateInfo', template_data)

    def update(self, request, context):
        params, metadata = self.parse_request(request, context)
        with self.locator.get_service('DashboardTemplateService', metadata) as template_svc:
            template_data = template_svc.update(params)
            return self.locator.get_info('DashboardTemplateInfo', template_data)

    def deregister(self, request, context):
        params, metadata = self.parse_request(request, context)
        with self.locator.get_service('DashboardTemplateService', metadata) as template_svc:
            template_svc.deregister(params)
            return self.locator.get_info('EmptyInfo')

    def enable(self, request, context):
        params, metadata = self.parse_request(request, context)
        with self.locator.get_service('DashboardTemplateService', metadata) as template_svc:
            template_data = template_svc.enable(params)
            return self.locator.get_info('DashboardTemplateInfo', template_data)

    def disable(self, request, context):
        params, metadata = self.parse_request(request, context)
        with self.locator.get_service('DashboardTemplateService', metadata) as template_svc:
            template_data = template_svc.disable(params)
            return self.locator.get_info('DashboardTemplateInfo', template_data)

    def get(self, request, context):
        params, metadata = self.parse_request(request, context)
        with self.locator.get_service('DashboardTemplateService', metadata) as template_svc:
            template_data = template_svc.get(params)
            return self.locator.get_info('DashboardTemplateInfo', template_data)

    def list(self, request, context):
        params, metadata = self.parse_request(request, context)
        with self.locator.get_service('DashboardTemplateService', metadata) as template_svc:
            templates_data, total_count = template_svc.list(params)
            return self.locator.get_info('DashboardTemplatesInfo', templates_data, total_count, minimal=self.get_minimal(params))




import logging
from spaceone.repository.error import *
from spaceone.repository.manager.dashboard_template_manager import DashboardTemplateManager

__all__ = ["ManagedDashboardTemplateManager"]

_LOGGER = logging.getLogger(__name__)


class ManagedDashboardTemplateManager(DashboardTemplateManager):

    def get_template(self, repo_info: dict, template_id: str, domain_id: str):
        raise ERROR_NO_DASHBOARD_TEMPLATE(template_id=template_id)

    def list_templates(self, repo_info: dict, query: dict, params: dict):
        return [], 0



import logging

from spaceone.core.connector.space_connector import SpaceConnector

from spaceone.repository.manager.dashboard_template_manager import DashboardTemplateManager

__all__ = ["RemoteDashboardTemplateManager"]

_LOGGER = logging.getLogger(__name__)


class RemoteDashboardTemplateManager(DashboardTemplateManager):
    def get_template(self, repo_info: dict, template_id: str, domain_id: str = None):
        endpoint = repo_info["endpoint"]
        market_place_token = repo_info["token"]

        remote_repo_conn: SpaceConnector = self.locator.get_connector(
            "SpaceConnector", endpoint=endpoint, token=market_place_token
        )

        template_info: dict = remote_repo_conn.dispatch(
            "DashboardTemplate.get", {"template_id": template_id, "repository_id": "repo-local"}
        )

        return self.change_response(template_info, repo_info)

    def list_templates(self, repo_info: dict, query: dict, params: dict):
        try:
            endpoint = repo_info["endpoint"]
            market_place_token = repo_info["token"]

            _LOGGER.debug(f"[list_templates] Remote Repository endpoint: {endpoint}")

            remote_repo_conn: SpaceConnector = self.locator.get_connector(
                SpaceConnector, endpoint=endpoint, token=market_place_token
            )

            response = remote_repo_conn.dispatch(
                "DashboardTemplate.list", {"query": query, "repository_id": "repo-local"}
            )

            plugins_info = response.get("results", [])
            total_count = response.get("total_count", 0)
            results = []

            for template_info in plugins_info:
                results.append(self.change_response(template_info, repo_info))

            return results, total_count
        except Exception as e:
            _LOGGER.error(f"[list_templates] {e}")
            return [], 0



import functools
from spaceone.api.repository.v1 import dashboard_template_pb2, repository_pb2
from spaceone.core.pygrpc.message_type import *
from spaceone.core import utils

__all__ = ["DashboardTemplateInfo", "DashboardTemplatesInfo"]


def DashboardTemplateInfo(template_info: dict, minimal=False):
    info = {
        "template_id": template_info.get("template_id"),
        "name": template_info.get("name"),
        "state": template_info.get("state"),
        "template_type": template_info.get("template_type"),
    }

    if not minimal:
        info.update(
            {
                "dashboards": change_list_value_type(template_info.get("dashboards")),
                "tags": change_struct_type(template_info.get("tags")),
                "labels": change_list_value_type(template_info.get("labels")),
                "domain_id": template_info.get("domain_id"),
                "created_at": utils.datetime_to_iso8601(template_info.get("created_at")),
                "updated_at": utils.datetime_to_iso8601(template_info.get("updated_at")),
            }
        )

        if repository_info := template_info.get("repository_info"):
            info.update(
                {
                    "repository_info": repository_pb2.RepositoryInfo(
                        repository_id=repository_info.get("repository_id"),
                        name=repository_info.get("name"),
                        repository_type=repository_info.get("repository_type"),
                    )
                }
            )

    return dashboard_template_pb2.DashboardTemplateInfo(**info)


def DashboardTemplatesInfo(templates_info, total_count, **kwargs):
    return dashboard_template_pb2.DashboardTemplatesInfo(
        results=list(map(functools.partial(DashboardTemplateInfo, **kwargs), templates_info)),
        total_count=total_count,
    )



import copy
import logging
import re

from spaceone.core.service import *
from spaceone.core import config, utils

from spaceone.repository.error import *
from spaceone.repository.manager.dashboard_template_manager.local_dashboard_template_manager import (
    LocalDashboardTemplateManager,
)
from spaceone.repository.manager.repository_manager import RepositoryManager

_LOGGER = logging.getLogger(__name__)


@authentication_handler
@authorization_handler
@mutation_handler
@event_handler
class DashboardTemplateService(BaseService):
    resource = "DashboardTemplate"

    @transaction(permission="repository:DashboardTemplate.write", role_types=["DOMAIN_ADMIN"])
    @check_required(["name", "domain_id"])
    def register(self, params):
        """Register DashboardTemplate (local repo only)

        Args:
            params (dict): {
                'template_id': 'str',
                'name': 'str',              # required
                'template_type': 'str',
                'dashboards': 'str',
                'labels': 'list',
                'tags': 'dict',
                'domain_id': 'str'          # injected from auth
            }

        Returns:
            template_info (dict)
        """

        template_id = params.get("template_id")

        if template_id is None:
            template_id = utils.generate_id("template")

        template_mgr: LocalDashboardTemplateManager = self.locator.get_manager("LocalDashboardTemplateManager")

        # Only LOCAL repository can be registered
        repo_mgr: RepositoryManager = self.locator.get_manager("RepositoryManager")
        params["template_id"] = template_id
        params["repository"] = repo_mgr.get_local_repository()
        params["repository_id"] = params["repository"].get("repository_id")

        return template_mgr.create_template(params)

    @transaction(permission="repository:DashboardTemplate.write", role_types=["DOMAIN_ADMIN"])
    @check_required(["template_id", "domain_id"])
    def update(self, params):
        """Update DashboardTemplate. (local repo only)

        Args:
            params (dict): {
                'template_id': 'str',     # required
                'name': 'str',
                'template_type': 'str',
                'dashboards': 'str',
                'labels': 'list',
                'tags': 'dict'
                'domain_id': 'str'      # injected from auth
            }

        Returns:
            template_info (dict)
        """

        template_mgr: LocalDashboardTemplateManager = self.locator.get_manager("LocalDashboardTemplateManager")
        return template_mgr.update_template(params)

    @transaction(
        permission="repository:DashboardTemplate.write",
        role_types=["DOMAIN_ADMIN"],
    )
    @check_required(["template_id", "domain_id"])
    def enable(self, params):
        """Enable template (local repo only)

        Args:
            params (dict): {
                'template_id': 'str',     # required
                'domain_id': 'str'      # injected from auth
            }

        Returns:
            template_info (dict)
        """

        template_mgr: LocalDashboardTemplateManager = self.locator.get_manager("LocalDashboardTemplateManager")
        return template_mgr.enable_template(params["template_id"], params["domain_id"])

    @transaction(
        permission="repository:DashboardTemplate.write",
        role_types=["DOMAIN_ADMIN"],
    )
    @check_required(["template_id", "domain_id"])
    def disable(self, params):
        """Disable DashboardTemplate (local repo only)

        Args:
            params (dict): {
                'template_id': 'str',     # required
                'domain_id': 'str'      # injected from auth
            }

        Returns:
            template_info (dict)
        """
        template_mgr: LocalDashboardTemplateManager = self.locator.get_manager("LocalDashboardTemplateManager")
        return template_mgr.disable_template(params["template_id"], params["domain_id"])

    @transaction(permission="repository:DashboardTemplate.write", role_types=["DOMAIN_ADMIN"])
    @check_required(["template_id", "domain_id"])
    def deregister(self, params):
        """Deregister DashboardTemplate (local repo only)

        Args:
            params (dict): {
                'template_id': 'str',
                'domain_id': 'str'
            }

        Returns:
            None
        """

        template_mgr: LocalDashboardTemplateManager = self.locator.get_manager("LocalDashboardTemplateManager")
        return template_mgr.delete_template(params["template_id"], params["domain_id"])

    @transaction(
        permission="repository:DashboardTemplate.read",
        role_types=["DOMAIN_ADMIN", "WORKSPACE_OWNER", "WORKSPACE_MEMBER", "USER"],
    )
    @check_required(["template_id"])
    @change_only_key({"repository_info": "repository"})
    def get(self, params):
        """Get DashboardTemplate (all repositories)

        Args:
            params (dict): {
                'template_id': 'str',       # required
                'repository_id': 'str',
                'domain_id': 'str',         # injected from auth (optional)
            }

        Returns:
            template_info (dict)
        """

        template_id = params["template_id"]
        repo_id = params.get("repository_id")
        domain_id = params.get("domain_id")

        repo_mgr: RepositoryManager = self.locator.get_manager("RepositoryManager")
        repos_info = repo_mgr.get_repositories(repo_id)
        for repo_info in repos_info:
            _LOGGER.debug(
                f"[get] find at name: {repo_info['name']} "
                f"(repo_type: {repo_info['repository_type']})"
            )

            template_mgr = self._get_template_manager_by_repo(repo_info["repository_type"])
            try:
                return template_mgr.get_template(repo_info, template_id, domain_id)
            except Exception as e:
                _LOGGER.debug(
                    f"[get] Can not find template({template_id}) at {repo_info['name']}"
                )

        raise ERROR_NO_DASHBOARD_TEMPLATE(template_id=template_id)

    @transaction(
        permission="repository:DashboardTemplate.read",
        role_types=["DOMAIN_ADMIN", "WORKSPACE_OWNER", "WORKSPACE_MEMBER", "USER"],
    )
    @change_only_key({"repository_info": "repository"}, key_path="query.only")
    @append_query_filter(
        ["template_id", "name", "state", "dashboard_type"]
    )
    def list(self, params):
        """List templates (all repositories)

        Args:
            params (dict): {
                'repository_id': 'str',
                'template_id': 'str',
                'name': 'str',
                'state': 'str',
                'dashboard_type': 'str',
                'domain_id': 'str',
                'query': 'dict (spaceone.api.core.v1.Query)'
            }

        Returns:
            results (list): 'list of template_info'
            total_count (int)
        """

        query = params.get("query", {})
        query["only"] = query.get("only", [])

        repo_mgr: RepositoryManager = self.locator.get_manager("RepositoryManager")
        if repository_id := params.get("repository_id"):
            repo_info = repo_mgr.get_repository(repository_id)
            template_mgr = self._get_template_manager_by_repo(repo_info["repository_type"])

            return template_mgr.list_templates(repo_info, query, params)
        else:
            all_templates_info = []
            template_total_count = 0
            query_page = None
            if "page" in query:
                query_page = query["page"]
                del query["page"]

            repos_info: list = repo_mgr.get_repositories()
            for repo_info in repos_info:
                template_mgr = self._get_template_manager_by_repo(
                    repo_info["repository_type"]
                )
                templates_info, total_count = template_mgr.list_templates(
                    repo_info, copy.deepcopy(query), params
                )
                all_templates_info += templates_info
                template_total_count += total_count

            if query_page:
                page_start = query_page.get("start", 1) - 1
                page_limit = query_page.get("limit")

                if page_limit:
                    all_templates_info = all_templates_info[
                        page_start : page_start + page_limit
                    ]

            return all_templates_info, template_total_count

    def _get_template_manager_by_repo(self, repository_type: str):
        if repository_type == "LOCAL":
            return self.locator.get_manager("LocalDashboardTemplateManager")
        elif repository_type == "MANAGED":
            return self.locator.get_manager("ManagedDashboardTemplateManager")
        else:
            return self.locator.get_manager("RemoteDashboardTemplateManager")



from spaceone.core.error import *


class ERROR_NO_DASHBOARD_TEMPLATE(ERROR_INVALID_ARGUMENT):
    _message = 'Dashboard Template does not exists. (template_id = {template_id})'


class ERROR_MANAGED_REPOSITORY_NOT_SUPPORT_LIST(ERROR_INVALID_ARGUMENT):
    _message = 'Managed Repository does not support list method.'



from mongoengine import *

from spaceone.core.model.mongo_model import MongoModel

__all__ = ["DashboardTemplate"]


class DashboardTemplate(MongoModel):
    template_id = StringField(max_length=255, unique_with="domain_id")
    name = StringField(max_length=255)
    state = StringField(
        max_length=40, default="ENABLED", choices=("ENABLED", "DISABLED")
    )
    template_type = StringField(max_length=255, default="SINGLE")
    dashboards = ListField(DictField())
    labels = ListField(StringField(max_length=255))
    tags = DictField()
    domain_id = StringField(max_length=255)
    created_at = DateTimeField(auto_now_add=True)
    updated_at = DateTimeField(auto_now=True)

    meta = {
        "updatable_fields": [
            "name",
            "state",
            "template_type",
            "dashboards",
            "labels",
            "tags",
        ],
        "minimal_fields": [
            "template_id",
            "name",
            "state",
            "template_type",
        ],
        "ordering": ["name"],
        "indexes": [
            "template_id",
            "state",
            "template_type",
            "domain_id",
        ],
    }



from spaceone.repository.info.common_info import *
from spaceone.repository.info.plugin_info import *
from spaceone.repository.info.dashboard_template_info import *
from spaceone.repository.info.repository_info import *



import logging

from spaceone.repository.model.dashboard_template_model import DashboardTemplate
from spaceone.repository.manager.dashboard_template_manager import DashboardTemplateManager
from spaceone.repository.manager.repository_manager import RepositoryManager

__all__ = ["LocalDashboardTemplateManager"]

_LOGGER = logging.getLogger(__name__)


class LocalDashboardTemplateManager(DashboardTemplateManager):
    def __init__(self, *args, **kwargs):
        super().__init__(*args, **kwargs)
        self.template_model: DashboardTemplate = self.locator.get_model("DashboardTemplate")
        self.repo_info = RepositoryManager.get_repositories(repository_type="LOCAL")[0]

    def create_template(self, params: dict):
        def _rollback(vo: DashboardTemplate):
            vo.delete()

        template_vo: DashboardTemplate = self.template_model.create(params)
        self.transaction.add_rollback(_rollback, template_vo)

        template_info = template_vo.to_dict()
        return self.change_response(template_info, self.repo_info)

    def update_template(self, params):
        template_vo = self.template_model.get(
            template_id=params["template_id"], domain_id=params["domain_id"]
        )
        return self.update_template_by_vo(params, template_vo)

    def enable_template(self, template_id, domain_id):
        template_vo = self.template_model.get(template_id=template_id, domain_id=domain_id)
        return self.update_template_by_vo({"state": "ENABLED"}, template_vo)

    def disable_template(self, template_id, domain_id):
        template_vo = self.template_model.get(template_id=template_id, domain_id=domain_id)
        return self.update_template_by_vo({"state": "DISABLED"}, template_vo)

    def update_template_by_vo(self, params, template_vo):
        def _rollback(old_data):
            _LOGGER.info(
                f'[ROLLBACK] Revert DashboardTemplate Data : {old_data["name"]} ({old_data["template_id"]})'
            )
            template_vo.update(old_data)

        self.transaction.add_rollback(_rollback, template_vo.to_dict())
        template_vo = template_vo.update(params)
        template_info = template_vo.to_dict()
        return self.change_response(template_info, self.repo_info)

    def delete_template(self, template_id, domain_id):
        template_vo = self.template_model.get(template_id=template_id, domain_id=domain_id)
        template_vo.delete()

    def get_template(self, repo_info: dict, template_id, domain_id: str):
        template_vo = self.template_model.get(template_id=template_id, domain_id=domain_id)

        template_info = template_vo.to_dict()
        return self.change_response(template_info, self.repo_info)

    def list_templates(self, repos_info: dict, query: dict, params: dict):
        domain_id = params.get("domain_id")
        keyword = query.get("keyword")
        query_filter = query.get("filter", [])
        query_filter_or = query.get("filter_or", [])
        query["filter"] = self._append_domain_filter(query_filter, domain_id)
        query["filter_or"] = self._append_keyword_filter(query_filter_or, keyword)
        if "repository" in query.get("only", []):
            query["only"].remove("repository")

        template_vos, total_count = self.template_model.query(**query)
        results = []
        for template_vo in template_vos:
            template_info = template_vo.to_dict()
            results.append(self.change_response(template_info, repos_info))

        return results, total_count

    @staticmethod
    def _append_domain_filter(query_filter, domain_id=None):
        if domain_id:
            query_filter.append({"k": "domain_id", "v": domain_id, "o": "eq"})

        return query_filter

    @staticmethod
    def _append_keyword_filter(query_filter_or, keyword):
        if keyword:
            query_filter_or.append({"k": "name", "v": keyword, "o": "contain"})

        return query_filter_or
